import asyncio
import functools
from functools import lru_cache
from io import BytesIO

import aiohttp
import requests
from aiocache import cached
from PIL import Image


async def run_in_executor(func, **kwargs):
    """Run function in executor

    Parameters
    ----------
    func : func
        Function to run
    """
    func = functools.partial(func, **kwargs)
    data = await asyncio.get_event_loop().run_in_executor(None, func)
    return data


@lru_cache(maxsize=32)
def load_image(link: str) -> Image.Image:
    """Load image from link

    Parameters
    ----------
    link : str
        Image link

    Returns
    -------
    PIL.Image.Image
        Image from the provided link (if any)
    """
    _bytes = BytesIO(requests.get(link).content)
    image = Image.open(_bytes).convert("RGBA")

    return image


@cached(ttl=60 * 60 * 24)
async def load_image_async(link: str, session=None) -> Image.Image:
    """Load image from link (async)

    Parameters
    ----------
    link : str
        Image from the provided link (if any)

    Returns
    -------
    PIL.Image.Image
        Image link
    """
    if isinstance(session, aiohttp.ClientSession):
        async with session.get(link) as response:  # type: ignore
            data = await response.read()
    else:
        async with aiohttp.ClientSession() as session:
            async with session.get(link) as response:
                data = await response.read()

    _bytes = BytesIO(data)
    image = Image.open(_bytes).convert("RGBA")
    return image



import os
import unittest
from io import BytesIO

from PIL import Image

from easy_pil import Canvas, Editor, Font, Text


class TestEditor(unittest.TestCase):
    def test_from_canvas(self):
        """Tests editor from canvas"""
        canvas = Canvas((100, 100), color="black")
        editor = Editor(canvas)
        self.assertIsInstance(editor, Editor)

    def test_from_path(self):
        """Tests editor from path"""
        editor = Editor(
            os.path.join(os.getcwd(), "examples", "assets", "pfp.png")
        )
        self.assertIsInstance(editor, Editor)

    def test_from_image(self):
        """Tests editor from image"""
        image = Image.open(
            os.path.join(os.getcwd(), "examples", "assets", "pfp.png")
        )
        editor = Editor(image)
        self.assertIsInstance(editor, Editor)

    def test_from_editor(self):
        """Tests editor from canvas"""
        canvas = Canvas((100, 100), color="black")
        editor1 = Editor(canvas)
        editor2 = Editor(editor1)
        self.assertEqual(type(editor1), type(editor2))

    def test_text(self):
        """Tests editor text"""
        canvas = Canvas((100, 100), color="black")
        editor = Editor(canvas).text(
            (50, 50), "Hello World", color="white", font=Font.poppins(size=20)
        )
        self.assertIsInstance(editor, Editor)

    def test_circle(self):
        """Tests editor circle"""
        canvas = Canvas((100, 100), color="black")
        editor = Editor(canvas).circle_image()
        self.assertIsInstance(editor, Editor)

    def test_rounded_corners(self):
        """Tests editor rounded corners"""
        canvas = Canvas((100, 100), color="black")
        editor = Editor(canvas).rounded_corners(radius=10, offset=5)
        self.assertIsInstance(editor, Editor)

    def test_resize(self):
        """Tests editor resize"""
        canvas = Canvas((100, 100), color="black")
        editor = Editor(canvas).resize((100, 50), crop=True)
        self.assertIsInstance(editor, Editor)

    def test_rotate(self):
        """Tests editor rotate"""
        canvas = Canvas((100, 100), color="black")
        editor = Editor(canvas).rotate(45)
        self.assertIsInstance(editor, Editor)

    def test_blur(self):
        """Tests editor blur"""
        canvas = Canvas((100, 100), color="black")
        editor = Editor(canvas).blur(mode="gaussian", amount=10)
        self.assertIsInstance(editor, Editor)

    def test_blend(self):
        """Tests editor blend"""
        canvas = Canvas((100, 100), color="black")
        canvas2 = Canvas((100, 100), color="red")
        editor = Editor(canvas).blend(canvas2, alpha=1, on_top=True)
        self.assertIsInstance(editor, Editor)

    def test_paste(self):
        """Tests editor paste"""
        canvas = Canvas((100, 100), color="black")
        canvas2 = Canvas((50, 50), color="red")
        editor = Editor(canvas).paste(canvas2, (0, 0))
        self.assertIsInstance(editor, Editor)

    def test_multi_text(self):
        """Tests editor multi text"""
        canvas = Canvas((200, 100), color="black")
        hello = Text("Hello ", color="white", font=Font.poppins(size=20))
        world = Text("World", color="white", font=Font.poppins(size=20))
        editor = Editor(canvas).multi_text(
            (0, 0), [hello, world], space_separated=False, align="left"
        )
        self.assertIsInstance(editor, Editor)

    def test_rectangle(self):
        """Tests editor rectangle"""
        canvas = Canvas((100, 100), color="black")
        editor = Editor(canvas).rectangle((10, 10), 80, 10, color="white")
        self.assertIsInstance(editor, Editor)

    def test_bar(self):
        """Tests editor bar"""
        canvas = Canvas((100, 100), color="black")
        editor = Editor(canvas).bar(
            (10, 10),
            80,
            10,
            50,
            color="white",
            outline="black",
            stroke_width=2,
            radius=5,
        )
        self.assertIsInstance(editor, Editor)

    def test_rounded_bar(self):
        """Tests editor rounded bar"""
        canvas = Canvas((100, 100), color="black")
        editor = Editor(canvas).rounded_bar(
            (10, 10), 80, 80, 50, color="white", stroke_width=2
        )
        self.assertIsInstance(editor, Editor)

    def test_arc(self):
        """Tests editor arc"""
        canvas = Canvas((100, 100), color="black")
        editor = Editor(canvas).arc(
            (10, 10), 80, 80, 0, 90, color="white", stroke_width=2
        )
        self.assertIsInstance(editor, Editor)

    def test_polygon(self):
        """Tests editor polygon"""
        canvas = Canvas((100, 100), color="black")
        cords = [(10, 10), (90, 10), (90, 90), (10, 90)]
        editor = Editor(canvas).polygon(cords, color="white", outline="black")
        self.assertIsInstance(editor, Editor)

    def test_bytes(self):
        """Tests editor bytes"""
        canvas = Canvas((100, 100), color="black")
        editor = Editor(canvas).image_bytes
        self.assertIsInstance(editor, BytesIO)


if __name__ == "__main__":
    unittest.main()



import unittest

from PIL import Image

from easy_pil import load_image, load_image_async


class TestUtils(unittest.IsolatedAsyncioTestCase):
    url: str = (
        "https://upload.wikimedia.org/wikipedia/commons/thumb/"
        "f/f9/Flag_of_Bangladesh.svg/800px-Flag_of_Bangladesh.svg.png"
    )

    def test_load_image(self):
        img = load_image(self.url)
        self.assertIsInstance(img, Image.Image)

    async def test_load_image_async(self):
        img = await load_image_async(self.url)
        self.assertIsInstance(img, Image.Image)

    async def test_cache(self):
        img1 = await load_image_async(self.url)
        img2 = await load_image_async(self.url)

        self.assertIsInstance(img1, Image.Image)
        self.assertIsInstance(img2, Image.Image)

        img3 = load_image(self.url)
        img4 = load_image(self.url)

        self.assertIsInstance(img3, Image.Image)
        self.assertIsInstance(img4, Image.Image)


if __name__ == "__main__":
    unittest.main()



from importlib.util import module_from_spec, spec_from_file_location
from pathlib import Path

from setuptools import find_packages, setup

current_directory = Path(__file__).parent.resolve()


def _get_requirements() -> list:
    with open(current_directory / "requirements.txt") as f:
        return f.read().splitlines()


def _get_long_desc() -> str:
    return (current_directory / "README.md").read_text(encoding="utf-8")


def _get_version() -> str:
    vpath = current_directory / "easy_pil" / "_version.py"
    spec = spec_from_file_location(vpath.name[:-3], vpath)
    mod = module_from_spec(spec)  # type: ignore
    spec.loader.exec_module(mod)  # type: ignore

    return mod.__version__


setup(
    name="easy-pil",
    version=_get_version(),
    description="A library to make common tasks of Pillow easy.",  # Optional
    long_description=_get_long_desc(),
    long_description_content_type="text/markdown",
    url="https://github.com/shahriyardx/easy-pil",
    author="Md Shahriyar Alam",
    author_email="contact@shahriyar.dev",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    keywords=(
        "Pillow, PIL, Pillow wrapper, PIL wrapper, Easy Pillow, "
        "Easy PIL, discord rank card, discord card"
    ),
    packages=find_packages(),
    package_data={
        "easy_pil": ["fonts/*/*.ttf"],
    },
    python_requires=">=3.7, <4",
    install_requires=_get_requirements(),
    extras_require={
        "dev": ["black", "isort", "click", "twine"],
    },
    project_urls={
        "Documentation": "https://easy-pil.readthedocs.io/en/latest/",
        "Bug Reports": "https://github.com/shahriyardx/easy-pil/issues",
        "Source": "https://github.com/shahriyardx/easy-pil/",
    },
)



import unittest

from easy_pil import Canvas


class TestCanvas(unittest.TestCase):
    def test_canvas(self):
        """Tests canvas"""
        canvas = Canvas((100, 100), color="black")
        canvas2 = Canvas(width=100, height=100, color="black")
        self.assertEqual(canvas.size, (100, 100))
        self.assertEqual(canvas2.size, (100, 100))


if __name__ == "__main__":
    unittest.main()
